#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import date, timedelta
import sys
import pandas as pd
import codecs
from .jwislib import JWIS

sys_encoding = sys.stdout.encoding

def ask_date():
    print("Beginning date")
    d_start_year = int(input("    year? "))
    d_start_month = int(input("    month? "))
    d_start_date = int(input("    date? "))
    d_start = date(d_start_year, d_start_month, d_start_date)

    print("Final date")
    d_end_year = int(input("    year? "))
    d_end_month = int(input("    month? "))
    d_end_date = int(input("    date? "))
    d_end = date(d_end_year, d_end_month, d_end_date)

    return (d_start, d_end)

def ask_observatory():
    obs_id = input("Type observatory ID: ")
    return obs_id

def main():
    date_periods = ask_date()
    d_start = date_periods[0]
    d_end = date_periods[1]
    if d_start > d_end:
        d_start, d_end = d_end, d_start

    obs_id = ask_observatory()

    output_filename = input("saving file name? ")
    jwis = JWIS(obs_id, d_start, d_end, "NO")
    jwis_table = jwis.retrieve_hq_data()
    jwis_table.to_csv(output_filename)
    print("Done")



# -*- coding: utf-8 -*-

# ------------------------------------------------
# JWISHTMLParser
#   HTML Parser for Japan Water Information System
# ------------------------------------------------

from urllib.parse import parse_qs
from html.parser import HTMLParser

class JWISParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.host = "http://www1.river.go.jp"

    def handle_starttag(self, tag, attrs):
        if tag == 'a':
            attrs_dict = dict(attrs)
            url = attrs_dict["href"]
            if url.startswith("/dat/dload/download"):
                self.data_url = self.host + url



# -*- coding: utf-8 -*-

# ------------------------------------------------
# jwislib
#   read & write data from Japan Water Information System
# ------------------------------------------------

import urllib.request
import urllib.parse
import datetime
import pandas as pd
from .JWISHTMLParser import JWISParser

class JWIS:
    def __init__(self, obs_id, date_begin, date_end, kawabou):
        self.view_url = "http://www1.river.go.jp/cgi-bin/DspWaterData.exe"
        self.obs_id = obs_id
        self.date_begin = date_begin
        self.date_end = date_end
        self.kawabou = kawabou

    def kind_name(self, kind):
        if kind == '1':
            return 'H'
        elif kind == '5':
            return 'Q'
        else:
            return 'X'

    def retrieve_data(self, kind):
        kn = self.kind_name(kind)
        columns = ["Date", "Time", kn, "Flag_" + kn]
        data = pd.DataFrame(columns=columns)

        url_params_dict = {
            "KIND": kind,
            "ID": self.obs_id,
            "KAWABOU": self.kawabou
        }

        d = self.date_begin
        while d <= self.date_end:
            date_delta = min(datetime.timedelta(days=30), self.date_end - d)
            d1 = d + date_delta
            url_params_dict["BGNDATE"] = d.strftime("%Y%m%d")
            url_params_dict["ENDDATE"] = d1.strftime("%Y%m%d")
            url_params = urllib.parse.urlencode(url_params_dict)
            view_uri = self.view_url + '?' + url_params
            f = urllib.request.urlopen(view_uri)

            parser = JWISParser()
            parser.feed(f.read().decode("euc-jp"))
            parser.close()

            data_list = []
            with urllib.request.urlopen(parser.data_url) as data_file:
                for line in data_file:
                    line = line.decode("Shift_JIS")
                    if line.count(',') == 3 and not line.startswith('#'):
                        data_list.append(line.rstrip("\r\n").split(','))

            data = data.append(pd.DataFrame(data_list, columns=columns))
            d = d1 + datetime.timedelta(days=1)
        return data

    def retrieve_hq_data(self):
        h_data = self.retrieve_data('1')
        q_data = self.retrieve_data('5')
        hq_data = pd.merge(h_data, q_data, on=["Date", "Time"], how="outer")
        return hq_data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
    setuptools_available = True
except ImportError:
    from distutils.core import setup
    setuptools_available = False


params = {}
if setuptools_available:
    params['entry_points'] = {'console_scripts': ['jwis = jwis:main']}
else:
    params['scripts'] = ['bin/jwis']


setup(
    name = "jwis",
    packages = ["jwis"],
    version = "0.1.1",
    description = "Data downloader from Japan Water System Information",
    author = "Kiwamu Ishikura",
    author_email = "ishikura.kiwamu@gmail.com",
    url = "https://github.org/i-kiwamu/jwis",
    keywords = ["download", "hydrology", "japan"],
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: Japanese",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
        "Topic :: Utilities",
        ],
    long_description = """\
jwis: Data downloader from Japan Water Information System
=========================================================

* Author: Kiwamu Ishikura
* Version: 0.1.0
* Licence: GPLv3

What's this?
------------
This is a program to get hydrological data from Water Information System provided from Japanese Ministry of Land, Infrastructure, Transport and Tourism (http://www1.river.go.jp/). You can retrieve data and save as csv.

Requirement
-----------
Python (>= 3.3)
*Beware that Python 2 can NOT work*

Usage
-----
Download jwis, and you can type the followings on your console::

```
$ python3 jwis.py
```

Program will ask you the beginning/final date of data you want, locations, and file name. Please follow the explanations.

If you put jwislib.py in your PYTHONLIB and put jwis.py in your PATH, you can use these program anywhere.
""",
    **params
)



#!/usr/bin/env python3

import sys

if __package__ is None and not hasattr(sys, "frozen"):
    # direct call of __main__.py
    import os.path
    path = os.path.realpath(os.path.abspath(__file__))
    sys.path.append(os.path.dirname(os.path.dirname(path)))

import jwis

if __name__ == '__main__':
    jwis.main()
